from datetime import datetime
import pandas as pd
import os
from csv import writer
from csv import reader

import unidecode as unidecode


class Parser:
    def __init__(self):
        self.data = None
        self.logs_directory = "../logs/"
        self.results_file = "../results/parsed_no_validation.csv"
        self.results_edited = "../results/parsed_no_validation_edited.csv"

    #reads a .log file and returns it parsed as a table row
    def readLog(self, fileName):
        row = [""] * len(self.data.columns)

        row[0] = fileName.replace(".log", "")

        with open(self.logs_directory + fileName) as fp:
            Lines = fp.readlines()
            comments1stcr = ""
            # comments2ndcr = ""
            checklistItems = ""
            position = ""
            startedExperiment = False

            for line in Lines:
                # auf is an array containing [0] = timestamp, [1] = category name, [2] = value
                aux = line.split(";")
                column = -1
                try:
                    column = int(self.data.columns.get_loc(aux[1]))
                except:
                    pass

                content = ""
                # print(aux)
                if aux[1] == "start":
                    if aux[2].strip() == "survey":
                        column = int(self.data.columns.get_loc("dateSurvey"))
                    elif aux[2].strip() == "cr-experiment-final":
                        column = int(self.data.columns.get_loc("dateCRFinal"))
                    elif aux[2].strip() == "dem_questions":
                        column = int(self.data.columns.get_loc("dateDem"))
                    date = aux[0]
                    if not "." in date:
                        date = date[:10] + "." + date[10:]
                    content = str(datetime.fromtimestamp(float(date)))

                elif aux[1].startswith("read_instructions"):
                    startedExperiment = True
                    column = int(self.data.columns.get_loc("cr1Start"))
                    date = aux[0]
                    if not "." in date:
                        date = date[:10] + "." + date[10:]
                    content = str(datetime.fromtimestamp(float(date)))

                elif aux[1].startswith("end_cr_experiment"):
                    column = int(self.data.columns.get_loc("cr1end"))
                    date = aux[0]
                    if not "." in date:
                        date = date[:10] + "." + date[10:]
                    content = str(datetime.fromtimestamp(float(date)))

                elif aux[1].startswith("question_demo4"):
                    position += aux[2].strip() + " + "

                elif aux[1].startswith("question_interruption_time"):
                    column = int(self.data.columns.get_loc("question_interruption_time"))
                    content = aux[2]

                elif aux[1] == "end":
                    column = int(self.data.columns.get_loc("dateFinished"))
                    date = aux[0]
                    if not "." in date:
                        date = date[:10] + "." + date[10:]
                    content = str(datetime.fromtimestamp(float(date)))

                elif aux[1] == "addComment" or aux[1] == "deletedComment" or aux[1] == "updateComment" or \
                        aux[1] == "cancelComment":
                    comments1stcr += aux[2]

                elif aux[1].startswith("checklist_items_skipped"):
                    column = int(self.data.columns.get_loc("checklist_items_skipped_comment"))
                    content = aux[2]

                elif not column == -1:
                    content = ""
                    if len(aux) == 3:
                        content = aux[2].strip()

                else:
                    pass
                    #print("column not found: ", aux[1])

                if not startedExperiment:
                    row[column] = content.strip()
                elif not row[column]:
                    if content.strip() == "Test" or content.strip() == "test":
                        content = ""
                    row[column] = content.strip()

                # print(aux[1], content)

        row[int(self.data.columns.get_loc("comments-experiment"))] = comments1stcr.replace("\n", " ")
        # row[int(self.data.columns.get_loc("comments-experiment2nd"))] = comments2ndcr
        row[int(self.data.columns.get_loc("question_demo4"))] = position[:-2]

        # row[int(self.data.columns.get_loc("VALIDATION_sde"))] = \
        #     int(validation[validation['participant'] == row[0]]['VALIDATION_sde'].item())
        #
        # row[int(self.data.columns.get_loc("VALIDATION_ckv"))] = \
        #     int(validation[validation['participant'] == row[0]]['VALIDATION_ckv'].item())

        return row

    def openTable(self, fileName):
        return pd.read_csv(fileName)

    def parse(self, fileName):
        # load the participants' logs already parsed
        self.data = self.openTable(self.results_file)

        # the participant id is their log filename
        participant = fileName.replace(".log", "")

        print("fileName:", fileName)
        print("participant:", participant)

        # to_remove = validation[validation['participant'] == participant]['remove'].item()

        to_remove = "no"

        #parses the particiant's log only if it was not already parsed
        if not (self.data['participant'] == participant).any() and to_remove == "no":
            # print(participant)                     #keep track of which logs are being parsed

            row = self.readLog(fileName)           #parses the log into a row for the table

            self.data.loc[-1] = row                #placed the new row at the beginning of the table
            self.data.index = self.data.index + 1  #shifts the table index
            self.data = self.data.sort_index()     #organizes the tables indexes

            self.data.to_csv(self.results_file, index=False)  #saves the updated table


    def addColumnToResults(self):
        count = 0
        with open(self.results_file, 'r') as read_obj, open(self.results_edited, 'w', newline='') as write_obj:
            csv_reader = reader(read_obj)
            csv_writer = writer(write_obj)
            for row in csv_reader:
                skipped = 0
                if count is 0:
                    row.append("itemsSkipped")
                else:
                    if row[3] == "SC":
                        temp = [row[6], row[7], row[8], row[9], row[20], row[23], row[24]]
                        for r in temp:
                            if r == "":
                                skipped += 1
                    elif row[3] == "LC":
                        temp = row[6:28]
                        for r in temp:
                            if r == '':
                                skipped += 1
                    row.append(skipped)
                csv_writer.writerow(row)
                count += 1


    #creates the spreadsheet file to store the parsed logs
    def createResultsFile(self):
        self.finished = "participant,data_policy,cr1Start,checklistType,pageLoaded," \
                        "read_instructions," \
                        "Is user-supplied input properly validated sanitized and filtered?," \
                        "Are minimum and maximum password lengths enforced?," \
                        "Is untrusted input data constrained for type length format and range?," \
                        "Is the set of allowed characters for passwords limited?," \
                        "Are passwords hidden from the user administrator or system?," \
                        "Does the password expire over time?," \
                        "Does the session ID length have a minimum of 128 bits (16 bytes)?," \
                        "Are session IDs generated uniquely and provide at least 64 bits of entropy?," \
                        "Is all session data validated?," \
                        "Are session IDs of newly generated sessions issued to a user only after successful authentication?," \
                        "Does session destruction after logging out or timeout happen on both the server and browser?," \
                        "Do session cookies have a short expiration time?," \
                        "Are non-persistent cookies used for session management purposes?," \
                        "Are authentication cookies encrypted?," \
                        "Is sensitive data logged in a masked sanitized hashed or encrypted state?," \
                        "Are connection attempts logged?," \
                        "Is all data in-transit encrypted with secure protocols (e.g. TLS)?," \
                        "Are up-to-date and appropriately strong standard encryption algorithms protocols and keys used?," \
                        "Are passwords stored in an encrypted format?," \
                        "Are keys hardcoded into the source code?," \
                        "Is data outputted by the application correctly encoded?," \
                        "Are encrypted keys and data stored on separate systems?," \
                        "comments-experiment,cr1end,pageClosed,question_interruption_time," \
                        "dateCRFinal,sde_found,sde_found_nc,sde_missed,checklist_helped_sde," \
                        "checklist_helped_sde_comment,checklist_not_helped_sde_comment,ckv_found,ckv_found_nc,ckv_missed," \
                        "checklist_helped_ckv,checklist_helped_ckv_comment,checklist_not_helped_ckv_comment,checklist_length,checklist_detail," \
                        "checklist_usability,checklist_items_skipped_comment,dateSurvey,familiar,incidents,courses," \
                        "practice,question_courses,question_role,updated,responsibility,designing,coding,reviewing," \
                        "question_sec5,sld6,question_which_tools,question_how_manually,checkpref,checkrely,checkeff," \
                        "checkhelps,checkexp,question_sec1extra,aware,expertise,availability,checkused,cr,third-lib," \
                        "documentation,time,increase_aware,qsec_adopt_cr,bettertests,checkaccess,betterdocs,sec_libs," \
                        "secperson,toolsiden,pastvuln,more_time,question_sec4extra,dateDem,question_gender,question_1," \
                        "question_demo4,question_demo42,question_dem6,question_demo5,question_dem9,question_dem10," \
                        "question_dem8,professional,java,year-cr,year-db,year-sql,often-design,often-programming," \
                        "often-cr,question_feedback1,question_feedback41,question_feedback2," \
                        "question_feedback3,dateFinished,VALIDATION_sde,VALIDATION_ckv"
        header = self.finished
        results = open(self.results_file, "w")
        results.write(header)
        results.close()


parser = Parser()
path = "../logs/"

validation = pd.read_csv('../results/only_validation.csv')


files = os.listdir(path)

parser.createResultsFile()

for logFile in files:
    if logFile.endswith('.log'):
        parser.parse(logFile)

parser.addColumnToResults()  #Adds a new column named "SkippedItems" that counts the number of skipped items
