# install.packages("english")
# install.packages("scales")
# install.packages("tidyverse")
# install.packages("ggplot2")
# install.packages("xtable")
# install.packages("tidyverse")
# install.packages("Hmisc")
# install.packages("corrplot")
# install.packages("nnet")
# install.packages("reshape2")
# install.packages("foreign")
# install.packages("viridis")
# install.packages("hrbrthemes")
# install.packages("gridExtra")
# install.packages("sjPlot")
# install.packages("stargazer")
# install.packages("writexl")
#install.packages("readxl")

require(gridExtra)
require(foreign);require(lmtest);require(sandwich);require(oddsratio)
require(dplyr);require(effsize);require(car);require(pscl);require(pastecs)
require(MASS);require(nnet);library(effsize);library(epiR);library(ggplot2);
require(gridExtra)
if(!require(rcompanion)){install.packages("rcompanion")}
if(!require(vcd)){install.packages("vcd")}
if(!require(psych)){install.packages("psych")}
if(!require(DescTools)){install.packages("DescTools")}
if(!require(epitools)){install.packages("epitools")}
library(rcompanion)

library(rcompanion)
library("writexl")
library(english)
library(reshape)
library(ggpubr)
library(scales)
library(tidyr)
library(xtable)
library(tidyverse)
library(Hmisc)
library(corrplot)
library(haven)
library(viridis)
library(hrbrthemes)
library("readxl")
library(dplyr)

# the following only works with RStudio! Sets working directory to source file
# location
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# if you're using R standard, please run the following instead:
# setwd(getSrcDirectory()[1])

getwd()

#create the folders to save the latex files and the figures
dir.create("../results/figures/", showWarnings = FALSE)
dir.create("../results/latex/", showWarnings = FALSE)

results <- read_csv(
  # TODO Adjust path
  file = "../results/parsed_logs_edited.csv",
  col_names = TRUE)

# Rename Column and Count Distribution NC,SC,LC
setNames(as.data.frame(table(results$checklistType)), c("treatment","n"))

results = results %>% mutate(duration_CR = as.double(difftime(results$cr1end,results$cr1Start,units = "mins")))

#results <- results %>% filter(results$remove == "no")

results$updated[results$updated == "7"] <- 1 #we replace the not applicable answer for never
results$responsibility[results$responsibility == "7"] <- 1  #we replace the not applicable answer for never
results$designing[results$designing == "7"] <- 1  #we replace the not applicable answer for never
results$reviewing[results$reviewing == "7"] <- 1  #we replace the not applicable answer for never
results$availability[results$availability == "7"] <- 1 #we replace the not applicable answer for strongly disagree
results$availability[results$availability == "6"] <- 3 #we replace the don't know answer for neutral
results$cr[results$cr == "7" | results$cr == "6"] <- 3 #we replace the not applicable/don't know answer for neutral
results$`third-lib`[results$`third-lib` == "7" | results$`third-lib` == "6"] <- 3 #we replace the not applicable/don't know answer for neutral 
results$documentation[results$documentation == "7" | results$documentation == "6"] <- 3 #we replace the not applicable/don't know answer for neutral 
results$time[results$time == "7" | results$time == "6"] <- 3 #we replace the not applicable/don't know answer for neutral 
results$checkpref[results$checkpref == "7" | results$checkpref == "6"] <- 3 #we replace the not applicable/don't know answer for neutral
results$checkeff[results$checkeff == "7" | results$checkeff == "6"] <- 3 #we replace the not applicable/don't know answer for neutral
results$checkhelps[results$checkhelps == "7" | results$checkhelps == "6"] <- 3 #we replace the not applicable/don't know answer for neutral
results$checkexp[results$checkexp == "7"] <- 1 #we replace the not applicable answer for never
results$availability[results$checkused == "7"] <- 1 #we replace the not applicable answer for strongly disagree
results$availability[results$checkused == "6"] <- 3 #we replace the don't know answer for neutral

results$practice[is.na(results$practice)] <- 0

setNames(as.data.frame(table(results$checklistType)), c("treatment","n"))

# We add a new column called VALIDATION_sde which can take the following values
# 0=Neither SDE nor CKV was found
# 1=SDE was found
# 2=CKV was found
# 3=Both SDE and CKV were found
results<-results %>% mutate(vuln_found = case_when(
  VALIDATION_sde==1 & VALIDATION_ckv==1 ~ "none",
  VALIDATION_sde==0 & VALIDATION_ckv==1 ~ "sde",
  VALIDATION_sde==1 & VALIDATION_ckv==0 ~ "ckv",
  VALIDATION_sde==0 & VALIDATION_ckv==0 ~ "both"
))

## For convenience, we report here the question associated to each var
#  the following questions are from the file '../tool/templates/experiment_final.html'

# [0=yes,1=no]
# sde_found                Did you find the Sensitive Information Leakage vulnerability?
# checklist_helped_sde     Did the checklist help you in identifying the Sensitive Information Leakage vulnerability?
# ckv_found                Did you find the MD5 cryptographic hash algorithm vulnerability?
# checklist_helped_ckv     Did the checklist help you in identifying the MD5 cryptographic hash algorithm vulnerability?

# [0=too short,1=just right in length,2=too long]
# checklist_length         The checklist was...

# [0=too specific,1=just the right amount of detail,2=too general]
# checklist_detail         The checklist was...

# [0=easy to use,1=neither easy nor complicated to use,2=complicated to use]
# checklist_usability      The checklist was...

# If the user skipped 1 or multiple items in the checklist, he/she had to comment on it
# checklist_items_skipped_comment

## For convenience, we report here the question associated to each var
#  the following questions are from the file '../tool/templates/survey.html'

# [0=No interruption time,1=1-2 minutes,2=up to 5 minutes,3=up to 10 minutes,4=more than 10 minutes]
# question_interruption_time  How long have you been interrupted during the execution of the review?

# [0=no,1=yes]
# familiar           I am familiar with the concept of software vulnerabilities
# incidents          I have experience with security incidents
# courses            I have participated in security courses and/or training
# practice           I use what I have learned in practice

# [1=Never,2=Rarely,3=Occasionally,4=A moderate amount,5=A great deal,6=I don't know,7=Not Applicable]
# updated            I keep myself up to date with security information
# responsibility     It is part of my job responsibility to look for vulnerabilities (the name of this variable is switched with `designing` in the logs, but corrected by the parser)
# designing          I actively consider vulnerabilities when designing software     (the name of this variable is switched with `responsibility` in the logs, but corrected by the parser)
# coding             I actively consider vulnerabilities when coding
# reviewing          I actively look for vulnerabilities when reviewing

# [1=Strongly disagree,2=Disagree,3=Neutral,4=Agree,5=Strongly agree,6=I don't know,7=Not Applicable]
# Please answer the following questions regarding checklists
# checkpref          I prefer using checklists for code reviewing
# checkrely          Checklists are mainly for inexperienced code reviewers
# checkeff           Checklists improve my code review efficiency
# checkhelps         Checklists help in finding security issues

# [1=Never,2=Rarely,3=Occasionally,4=A moderate amount,5=A great deal,6=I don't know,7=Not Applicable]
# checkexp           I have used checklists for code review before

# [1=Strongly disagree,2=Disagree,3=Neutral,4=Agree,5=Strongly agree,6=I don't know,7=Not Applicable]
# Considering the project you work on the most, please rate your agreement with the following statements.
# aware              Project developers are aware of potential security issues
# expertise          Project developers have expertise in software security
# availability       Detection tools are used to find software vulnerabilities
# checkused          Checklists are used to find software vulnerabilites
# cr                 Code review is used to find software vulnerabilities
# third-lib          Third-party libraries are checked for security issues
# documentation      Security documentation and training are available
# time               Enough time is available to consider security aspects

# [1=Not at all effective,2=Slightly effective,3=Somewhat effective,4=Very effective,5=Extremely effective,6=I don't know,7=Not Applicable]
# Considering the project you work on the most, rate how effective, in your opinion, the following measures would be in reducing the number of vulnerabilities:
# increase_aware     Increasing awareness of potential security issues
# qsec_adopt_cr      Increasing the use of code review to find software vulnerabilities
# bettertests        Writing more and better tests
# checkaccess        Having access to security vulnerability checklists
# betterdocs         Having more and better security documentation and training
# sec_libs           Reinforcing the security check of third-party libraries
# secperson          Having people responsible for security in the project/company
# toolsiden          Increasing the use of tools to identify security bugs
# pastvuln           Learning from past vulnerabilities that affected the project
# more_time          more_time

#  the following questions are from the file '../tool/templates/dem_questions.html'

# text area
# question_gender    What is your gender?

# [0=High School but did not graduate,1=High School graduate or GED,2=Some college or 2-year degree,
#  3=Bachelor in CS (Computer Science),4=Bachelor in STEM (Science, Technology, Engineering and Mathematics),
#  5=Bachelor in a different area than CS or STEM,6=Master in CS,7=Master in STEM,8=Master in a different area than CS or STEM,
#  9=PhD or higher academic title in CS,10=PhD or higher academic title in STEM,
#  11=PhD or higher academic title in a different area than CS or STEM]
# question_1         What is your highest achieved level of education?

# [checkbox; 0=Student,1=Intern,2=Part-time position,3=Full-time position,4=Unemployed]
# question_demo4     What's your current employment status?
# [0=yes,1=no]
# question_dem6       Are you currently enrolled in a degree-granting program?
#
# [0=Software developer,1=Penetration tester,2=Bug bounty hunter,3=Software architect,
#  4=IT operations,5=Project manager,6=Researcher / Professor,7=PhD student]
# question_demo5      What is your current position related to?

# [0=yes,1=no]
# question_dem8      Do you contribute to open-source software systems as one of your main programming activities?

# [1=None,2=1 year or less,3=2 years,4=3-5 years,5=6-10 years,6=11 years or more]
# Please specify for how many years you have performed the following activities
#
# professional       Developed software in a professional setting (e.g., as employee of an IT-company or Open-Source contributor)
# java               Programmed in Java (also not professionally)
# year-cr            Reviewed code
# year-db            Worked with web applications
# year-sql           Worked with applications using databases

# [1=Not at all,2=About once,3=About once a month,4=About once a week,5=Daily or more often]
# Please specify how often you have performed the following activities in the last year
#
# often-design       Designed code
# often-programming  Developed code
# often-cr           Reviewed code

# [0=empty,1=yes,2=no,3=irrelevant]
# The following items are found in the Long Checklist with the correct answer attached to each item
# * indicating that the item is Part of the Short Checklist
#
# *Is user-supplied input properly validated, sanitized, and filtered? YES
# *Is untrusted input data constrained for type, length, format, and range? YES
# *Are minimum and maximum password lengths enforced? YES
# *Is the set of allowed characters for passwords limited? YES
# Are passwords hidden from the user, administrator, or system? YES (This question might be ambiguous, as hashed passwords are exposed but plaintext passwords are hidden)
# Does the password expire over time? NO or IRRELEVANT (Can be argued to be handeled in a different class not part of the code review)
# Does the session ID length have a minimum of 128 bits (16 bytes)? IRRELEVANT
# Are session IDs generated uniquely and provide at least 64 bits of entropy? IRRELEVANT
# Is all session data validated? IRRELEVANT
# Are session IDs of newly generated sessions issued to a user only after successful authentication? IRRELEVANT
# Does session destruction after logging out or timeout happen on both the server and browser? IRRELEVANT
# Do session cookies have a short expiration time? IRRELEVANT
# Are non-persistent cookies used for session management purposes? IRRELEVANT
# Are authentication cookies encrypted? IRRELEVANT
# Are connection attempts logged? NO or IRRELEVANT (Can be argued to be missing)
# *Is sensitive data logged in a masked, sanitized, hashed, or encrypted state? NO
# Is all data in-transit encrypted with secure protocols (e.g., TLS)? IRRELEVANT
# *Are up-to-date and appropriately strong standard encryption algorithms, protocols and keys used? NO
# *Are passwords stored in an encrypted format? YES
# Are keys hard-coded into the source code? NO or IRRELEVANT
# Are encrypted keys and data stored on separate systems? IRRELEVANT
# Is data outputted by the application correctly encoded? IRRELEVANT


# remove participants who did not add any comment in the first review
# this command excludes 8 participants

# Adds a new column named 'duration_CR'  which contain the minutes needed
# for the code review
results = results %>%   
  mutate( duration_CR = as.double(difftime(results$cr1end,results$cr1Start,units = "mins")))

boxplot(results$duration_CR)

#######################SDE#########################

###############

#knowledge
tbl = table(results$VALIDATION_sde, results$familiar) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_ckv, results$familiar) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_sde, results$courses) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_ckv, results$courses) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_sde, results$updated) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_ckv, results$updated) 
chisq.test(tbl) 
cramerV(tbl) 

#practice
wilcox.test(as.numeric(results$incidents)~results$VALIDATION_sde)
wilcox.test(as.numeric(results$incidents)~results$VALIDATION_ckv)

wilcox.test(as.numeric(results$responsibility)~results$VALIDATION_sde)
wilcox.test(as.numeric(results$responsibility)~results$VALIDATION_ckv)

wilcox.test(as.numeric(results$designing)~results$VALIDATION_sde)
wilcox.test(as.numeric(results$designing)~results$VALIDATION_ckv)

wilcox.test(as.numeric(results$coding)~results$VALIDATION_sde)
wilcox.test(as.numeric(results$coding)~results$VALIDATION_ckv)

wilcox.test(as.numeric(results$reviewing)~results$VALIDATION_sde)
wilcox.test(as.numeric(results$reviewing)~results$VALIDATION_ckv)

#demographics
wilcox.test(as.numeric(results$`often-cr`)~results$VALIDATION_sde) 
wilcox.test(as.numeric(results$`often-cr`)~results$VALIDATION_ckv) 

wilcox.test(as.numeric(results$`often-design`)~results$VALIDATION_sde) 
wilcox.test(as.numeric(results$`often-design`)~results$VALIDATION_ckv) 

wilcox.test(as.numeric(results$`often-programming`)~results$VALIDATION_sde) 
wilcox.test(as.numeric(results$`often-programming`)~results$VALIDATION_ckv) 

res.aov <- aov(java ~ checklistType, data = results)
summary(res.aov)
p <- TukeyHSD(res.aov)
p$checklistType
result<-data.frame(p$checklistType)
p<-result["p.adj"]
round(p.adjust(p$p.adj, method="holm"),3)

tbl = table(results$VALIDATION_sde, results$java) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$VALIDATION_ckv, results$java) 
chisq.test(tbl) 
cramerV(tbl) 

tbl = table(results$checklistType, results$java) 

chisq.test(tbl) 
cramerV(tbl) 

aux <- results %>% filter(results$checklistType == "NF" | results$checklistType == "NC")

count(aux)

tbl = table(aux$checklistType, aux$java) 
chisq.test(tbl) 
cramerV(tbl) 


cohen.d(as.numeric(aux$java),aux$checklistType)

wilcox.test(as.numeric(aux$java)~aux$checklistType) 



require(graphics)

set.seed(123)
x <- rnorm(50, mean = c(rep(0, 25), rep(3, 25)))
p <- 2*pnorm(sort(-abs(x)))

round(p, 3)
round(p.adjust(p), 3)
round(p.adjust(p, "BH"), 3)
